hook global WinSetOption filetype=python %{
	# set-option window expandtab true
	# set-option window softtabstop 4
	set-option window formatcmd "ruff format -"
	hook -group python-auto-format window BufWritePre .* %{
    	try format
	}
	hook -once -always WinSetOption filetype=.* %{
		unset-option window formatcmd
		remove-hooks window python-auto-format
	}
}

hook global WinSetOption filetype=toml %{
    set-option window formatcmd "taplo fmt -"
	# hook -group toml-auto-format window BufWritePre .* %{
 #    	try format
	# }
	# hook -once -always window WinSetOption filetype=.* %{
	# 	unset-option window formatcmd
	# 	remove-hooks window toml-auto-format
	# }
}

hook global WinSetOption filetype=latex %{
	set-option window formatcmd "tex-fmt --stdin"
	hook -group latex-auto-format window BufWritePre .* %{
    	try format
	}
	hook -once -always window WinSetOption filetype=.* %{
		unset-option window formatcmd
		remove-hooks window latex-auto-format
	}
}

hook global WinSetOption filetype=go %{
	# set-option window expandtab false
	# set-option window softtabstop 4
}

hook global WinSetOption filetype=kak %{
	set-option window formatcmd "%val{config}/scripts/kakfmt.pl"
	# set-option window expandtab false
}

hook global WinSetOption filetype=typst %{
	set-option buffer formatcmd "typstyle --wrap-text"
	hook -group typst-auto-format window BufWritePre .* format
	hook -once -always window WinSetOption filetype=.* %{
		unset-option window formatcmd
		remove-hooks window typst-auto-format
	}
        define-command -docstring %{
        	Spawns a Zathura pdf preview and Typst watcher for the currently open Typst file
        } typst %{
        	nop %sh{
        		{
        			"$kak_config/scripts/kak-typ-zathura.fish" -k -w "$kak_buffile" "$kak_client_pid"
        		} > /dev/null 2>&1 < /dev/null &
        	}
        }
}

hook global WinSetOption filetype=fish %{
	set-option window formatcmd "fish_indent"
}

# TODO: modify man.kak so that it leaves one char of whitespace
# on the rightmost column of the whole file, so that selecting the whole
# line doesn't cause hortizonal overflow!
hook global WinSetOption filetype=man %{
	try %{
		# remove these highlighters so everything displays properly
		remove-highlighter window/number-lines
		remove-highlighter window/show-whitespaces
		ui-wrap-disable
	}
	# man.kak emits this user hook event
	# This is needed because we can't modify the selection
	# in WinSetOption's context
	hook -once global User manpageloaded %{
		# This will move the cursor to the enter of the screen
		set-option window scrolloff "99,0"
		execute-keys %sh{
			result=$((kak_window_height / 2 - 1))
			printf '%d%s' "$result" 'j'
		}
	}
}

hook global WinSetOption filetype=kdl %{
    set-option window formatcmd "kdlfmt format -"
    hook -group kdl-auto-format window BufWritePre .* format
    hook -once -always WinSetOption filetype=.* %{
        unset-option window formatcmd
        remove-hooks window kdl-auto-format
    }
}

hook global WinSetOption filetype=mail %~
    evaluate-commands %sh{
        # stdin, we assume it's a pager
        if [ "$kak_bufname" = "*stdin*" ]; then
            echo 'define-command -hidden ismailreader nop'
        fi
    }
    try %{
        ismailreader
        set buffer readonly true
    	try %{
    		# remove these highlighters so everything displays properly
    		remove-highlighter window/number-lines
    		remove-highlighter window/show-whitespaces
    		ui-wrap-disable
    		ui-scrolloff-disable
    	}
    } catch %{
        set-option window formatcmd 'mailfmt'
        set-option window comment_line '>'
        try autospell-enable
        hook -group mail-auto-format window BufWritePre .* format
        hook -once -always window WinSetOption filetype=.* %{
            unset-option window formatcmd
            remove-hooks window mail-auto-format
        }
    }
~

hook global WinSetOption filetype=ini %{
    set-option window comment_line '#'
}

hook global BufCreate .*[.](conf) %{
	set-option buffer filetype ini
}

hook global BufCreate .*[.](jinja) %{
    set-option buffer filetype html
}


